package com.codigoparallevar.minicards.parts.connectors;

import android.util.Log;

import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.types.Drawable;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.Tuple2;
import com.codigoparallevar.minicards.types.connectors.Wiring.BooleanWire;
import com.codigoparallevar.minicards.types.connectors.input.BooleanInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.InputConnector;
import com.codigoparallevar.minicards.types.connectors.output.BooleanOutputConnector;
import com.codigoparallevar.minicards.types.wireData.BooleanSignal;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

public class BooleanRoundOutputConnector implements Drawable, BooleanOutputConnector {
    private PartGrid _partGrid;
    private int _centerX;
    private int _centerY;
    private final int _radius;
    private final Part _part;
    private BooleanWire _currentWire = null;
    private final List<BooleanWire> _wires;
    private final HashSet<InputConnector> _connections;

    public BooleanRoundOutputConnector(Part part, PartGrid partGrid, int centerX, int centerY, int radius) {
        _part = part;
        _partGrid = partGrid;
        _centerX = centerX;
        _centerY = centerY;
        _radius  = radius;
        _wires = new LinkedList<>();
        _connections = new HashSet<>();
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return ((Math.abs(x - _centerX) <= _radius)
                && (Math.abs(y - _centerY) <= _radius));
    }


    @Override
    public Moveable getMoveable() {
        if (_currentWire == null) {
            startWire();
        }

        return _currentWire;
    }

    @Override
    public void unlink() {}

    private void startWire() {
        _currentWire = new BooleanWire(this, _centerX, _centerY);
    }

    @Override
    public void drop(BooleanWire wire) {

        if (wire == _currentWire){
            _currentWire = null;

            BooleanInputConnector resultPoint = _partGrid.getBooleanInputConnectorOn(
                    wire.getXEnd(), wire.getYEnd());

            Log.d("RoundOutputConnector", "Dropped wire on " + resultPoint);

            // Not connected
            if (resultPoint == null){
                return;
            }

            // Already connected
            if (_connections.contains(resultPoint)) {
                return;
            }
            _connections.add(resultPoint);

            wire.attachTo(resultPoint);
            _wires.add(wire);
        }
        else {
            Log.w("RoundOutputConnector",
                    "Asked to drop non matching wire "
                            + "(expected " + _currentWire + ", got " + wire + ")");
        }
    }

    @Override
    public void unlinkWire(BooleanWire wire) {
        _wires.remove(wire);
    }

    @Override
    public void drawWires(ScrolledCanvas canvas, boolean devMode) {
        for (BooleanWire wire : _wires) {
            wire.draw(canvas, devMode);
        }

        if (_currentWire != null) {
            _currentWire.draw(canvas, devMode);
        }
    }

    @Override
    public void updatePosition(int x, int y) {
        _centerX = x;
        _centerY = y;

        for (BooleanWire wire : _wires){
            wire.moveStart(x, y);
        }
    }

    @Override
    public List<Tuple2<String, String>> getConnectionEndpoints() {
        List<Tuple2<String, String>> endpointIds = new LinkedList<>();

        for (BooleanWire wire : _wires) {
            InputConnector inputConnector = wire.getAttachedTo();
            Part endPart = inputConnector.getPart();
            endpointIds.add(new Tuple2<>(inputConnector.getId(), endPart.get_id()));
        }

        return endpointIds;
    }

    @Override
    public void connectTo(BooleanInputConnector inputConnector) {
        if (_connections.contains(inputConnector)) {
            return;
        }

        _connections.add(inputConnector);

        BooleanWire wire = new BooleanWire(this, _centerX, _centerY);
        wire.attachTo(inputConnector);
        _wires.add(wire);
    }

    @Override
    public void draw(ScrolledCanvas canvas, boolean devMode) {
        // TODO: Complete this part
    }

    public void send(BooleanSignal signal) {
        for (BooleanWire wire : _wires){
            wire.send(signal);
        }
    }
}
