package com.codigoparallevar.minicards.parts.samples;

import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.Log;

import com.codigoparallevar.minicards.PartInstantiator;
import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.parts.connectors.BooleanRoundInputConnector;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.Tuple2;
import com.codigoparallevar.minicards.types.connectors.input.BooleanInputConnector;
import com.codigoparallevar.minicards.types.connectors.input.InputConnector;
import com.codigoparallevar.minicards.types.connectors.output.OutputConnector;
import com.codigoparallevar.minicards.types.wireData.BooleanSignal;
import com.codigoparallevar.minicards.types.wireData.WireDataType;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

public class ColorBox implements Part {
    private static final String TOGGLE_INPUT_CONNECTOR_ID = "toggle_input_connector";
    private static final int DEFAULT_SIDE_SIZE = 100;
    private final String _id;
    private final PartGrid _partGrid;
    private boolean _activated;
    private int _left;
    private int _top;
    private int _right;
    private int _bottom;
    private List<InputConnector> inputConnectors;
    private BooleanInputConnector _toggleInputConnector;

    private ColorBox(String id, PartGrid partGrid, int left, int top, int right, int bottom) {
        _id = id;
        _partGrid = partGrid;
        _left = left;
        _top = top;
        _right = right;
        _bottom = bottom;
        _activated = false;

        _toggleInputConnector = new BooleanRoundInputConnector(
                this,
                getInputConnectorCenterX(),
                getInputConnectorCenterY(),
                getInputConnectRadius());
        inputConnectors = new LinkedList<>();
        inputConnectors.add(_toggleInputConnector);

    }

    public ColorBox(PartGrid partGrid, int left, int top, int right, int bottom) {
        this(UUID.randomUUID().toString(), partGrid, left, top, right, bottom);
    }

    @Override
    public int get_left() {
        return _left;
    }

    @Override
    public int get_right() {
        return _right;
    }

    @Override
    public int get_top() {
        return _top;
    }

    @Override
    public int get_bottom() {
        return _bottom;
    }

    @Override
    public void draw(ScrolledCanvas canvas, boolean devMode) {
        Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        if (_activated){
            paint.setColor(Color.GREEN);
        }
        else {
            paint.setColor(Color.RED);
        }

        if (devMode){
            drawConnector(canvas);
            // drawWires(canvas, devMode);
        }


        canvas.drawRect(
                new Rect(_left, _top,
                        _right, _bottom),
                paint);
    }

    private void drawConnector(ScrolledCanvas canvas) {
        Paint connectorPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        connectorPaint.setColor(Color.YELLOW);

        canvas.drawCircle(
                getInputConnectorCenterX(), getInputConnectorCenterY(),
                getInputConnectRadius(),
                connectorPaint);
    }

    @Override
    public void moveEnd(int x, int y) {
        final int width = _right - _left;
        final int height = _bottom - _top;

        _left = x - width / 2;
        _right = _left + width;

        _top = y - height / 2;
        _bottom = _top + height;

        _toggleInputConnector.updatePosition(
                getInputConnectorCenterX(),
                getInputConnectorCenterY());
    }

    @Override
    public void drop(int x, int y) {
        moveEnd(x, y);
    }

    @Override
    public void touched() {
        Log.d("Placeholder", "Placeholder touched");
    }

    @Override
    public List<InputConnector> getInputConnectors() {
        return inputConnectors;
    }

    @Override
    public List<OutputConnector> getOutputConnectors() {
        return Collections.emptyList();
    }

    public BooleanInputConnector getStateInputConnector() {
        return _toggleInputConnector;
    }

    @Override
    public JSONObject serialize() throws JSONException {
        JSONObject serialized = new JSONObject();

        serialized.put("id", _id);
        serialized.put("left", _left);
        serialized.put("top", _top);
        serialized.put("right", _right);
        serialized.put("bottom", _bottom);

        return serialized;
    }

    @Override
    public void send(InputConnector roundInputConnector, WireDataType signal) {
        if (signal instanceof BooleanSignal) {
            _activated = ((BooleanSignal) signal).value;
        }
    }

    @Override
    public String get_id() {
        return _id;
    }

    @Override
    public InputConnector getConnectorWithId(String inputConnectorId) {
        switch (inputConnectorId){
            case ColorBox.TOGGLE_INPUT_CONNECTOR_ID:
                return _toggleInputConnector;

            default:
                return null;
        }
    }

    @Override
    public String getConnectorId(InputConnector inputConnector) {
        if (inputConnector == _toggleInputConnector){
            return ColorBox.TOGGLE_INPUT_CONNECTOR_ID;
        }

        return null;
    }

    @Override
    public void resume() {

    }

    @Override
    public void pause() {

    }

    public static Part deserialize(PartGrid partGrid, JSONObject data) throws JSONException {
        String id = data.getString("id");
        int left = data.getInt("left");
        int top = data.getInt("top");
        int right = data.getInt("right");
        int bottom = data.getInt("bottom");

        return new ColorBox(id, partGrid, left, top, right, bottom);
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return (x >= get_left()) && (x <= get_right())
                && (y >= get_top()) && (y <= get_bottom());
    }

    @Override
    public Moveable getMoveable() {
        return this;
    }

    @Override
    public void unlink() {
        for (InputConnector input : getInputConnectors()) {
            input.unlink();
        }
    }

    public int getInputConnectorCenterX() {
        return get_left();
    }

    private int getInputConnectRadius() {
        return (get_right() - get_left()) / 2;
    }

    public int getInputConnectorCenterY() {
        return (get_top() + get_bottom()) / 2;
    }

    public static PartInstantiator getInstantiator() {
        final int halfSideSize = DEFAULT_SIDE_SIZE / 2;
        return new PartInstantiator() {
            @Override
            protected Part instantiate(PartGrid grid, Tuple2<Integer, Integer> center) {
                return new ColorBox(grid,
                        center.item1 - halfSideSize, center.item2 - halfSideSize,
                        center.item1 + halfSideSize, center.item2 + halfSideSize);
            }
        };
    }
}
